module Cli exposing (..)


type alias Parameter =
    { name : String
    }


type alias FilledParameter =
    { name : String
    , content : String
    }


type alias DirtyCommand =
    Command


type alias Command =
    { name : String
    , params : List Parameter
    }


type alias FilledCommand =
    { name : String
    , params : List FilledParameter
    }


type alias CommandErr =
    { command : Maybe Command
    , got : String
    , argumentErr : Maybe ArgumentErr
    }

type ArgumentErr
    = MissingArgs
    | TooManyArgs


parseInput : List Command -> String -> Result CommandErr FilledCommand
parseInput commands input =
    let
        tokens =
            List.map String.trim (String.split " " input)
    in
    case List.head tokens of
        Just maybeCommand ->
            case List.head (List.filter (isCommand maybeCommand) commands) of
                Just command ->
                    case applyArguments command.params tokens of
                        Ok list ->
                            Ok
                                { name = command.name
                                , params = list
                                }
                        Err err ->
                            Err { command = Just command
                                , got = input
                                , argumentErr = Just err
                                }


                Nothing ->
                    Err
                        { command = Nothing
                        , got = input
                        , argumentErr = Nothing
                        }
        Nothing ->
            Err
                { command = Nothing
                , got = input
                , argumentErr = Nothing
                }


applyArguments : List Parameter -> List String -> Result ArgumentErr (List FilledParameter)
applyArguments params inputs =
    if List.length params > List.length inputs then
        Err MissingArgs
    else if List.length params < List.length inputs then
        Err TooManyArgs
    else
        Ok (List.map2 applyParameter params inputs)



applyParameter : Parameter -> String -> FilledParameter
applyParameter param input =
    { name = param.name
    , content = input
    }
        

isCommand : String -> Command -> Bool
isCommand name command =
    if name == command.name then
        True

    else
        False
